@@ -168,10 +168,16 @@ public class AppointmentResource extends CalDAVResource<Appointment> {
     protected void saveObject(boolean checkPermissions) throws OXException {
         try {
             /*
-             * get original data
+             * load original appointment and change exceptions
              */
             Appointment originalAppointment = parent.load(this.object, false);
             CalendarDataObject[] originalExceptions = parent.loadChangeExceptions(this.object.getObjectID());
+            /*
+             * transform change- to delete-exceptions where user is removed from participants if needed (bug #26293)
+             */
+            if (null != originalExceptions && 0 < originalExceptions.length) {
+                originalExceptions = Patches.Outgoing.setDeleteExceptionForRemovedParticipant(factory, originalAppointment, originalExceptions);
+            }
             Date clientLastModified = this.object.getLastModified();
             if (clientLastModified.before(originalAppointment.getLastModified())) {
                 throw super.protocolException(HttpServletResponse.SC_CONFLICT);
@@ -199,6 +205,13 @@ public class AppointmentResource extends CalDAVResource<Appointment> {
              * update change exceptions
              */
             for (CalendarDataObject exceptionToSave : exceptionsToSave) {
+                /*
+                 * check if already deleted
+                 */
+                if (containsDeleteException(originalAppointment, exceptionToSave.getRecurrenceDatePosition())) {
+                    LOG.debug("Delete exception " + exceptionToSave + " already exists, skipping update.");
+                    continue;
+                }
                 Appointment originalException = getMatchingException(originalExceptions, exceptionToSave.getRecurrenceDatePosition());
                 if (null != originalException) {
                     /*
@@ -237,8 +250,6 @@ public class AppointmentResource extends CalDAVResource<Appointment> {
                     getAppointmentInterface().updateAppointmentObject(exceptionToSave, parentFolderID, clientLastModified, checkPermissions);
                     clientLastModified = exceptionToSave.getLastModified();
                 }
-//                getAppointmentInterface().updateAppointmentObject(exceptionToSave, parentFolderID, clientLastModified, checkPermissions);
-//                clientLastModified = exceptionToSave.getLastModified();
             }
             /*
              * update delete exceptions
@@ -321,21 +332,28 @@ public class AppointmentResource extends CalDAVResource<Appointment> {
         final List<ConversionError> conversionErrors = new LinkedList<ConversionError>();
         final List<ConversionWarning> conversionWarnings = new LinkedList<ConversionWarning>();
         try {
+            /*
+             * load appointment and change exceptions
+             */
+            CalendarDataObject appointment = parent.load(object, true);
+            CalendarDataObject[] changeExceptions = 0 < object.getRecurrenceID() ? parent.getChangeExceptions(object.getObjectID()) : null;
+            /*
+             * transform change- to delete-exceptions where user is removed from participants if needed (bug #26293)
+             */
+            if (null != changeExceptions && 0 < changeExceptions.length) {
+                changeExceptions = Patches.Outgoing.setDeleteExceptionForRemovedParticipant(factory, appointment, changeExceptions);
+            }
             /*
              * write appointment
              */
-            icalEmitter.writeAppointment(session, parent.load(object, true),
-                factory.getContext(), conversionErrors, conversionWarnings);
-            if (0 < object.getRecurrenceID()) {
-                CalendarDataObject[] changeExceptions = parent.getChangeExceptions(object.getObjectID());
-                if (null != changeExceptions && 0 < changeExceptions.length) {
-                    /*
-                     * write exceptions
-                     */
-                    for (Appointment changeException : changeExceptions) {
-                        icalEmitter.writeAppointment(session, parent.load(changeException, true),
-                            factory.getContext(), conversionErrors, conversionWarnings);
-                    }
+            icalEmitter.writeAppointment(session, appointment, factory.getContext(), conversionErrors, conversionWarnings);
+            /*
+             * write exceptions
+             */
+            if (null != changeExceptions && 0 < changeExceptions.length) {
+                for (Appointment changeException : changeExceptions) {
+                    icalEmitter.writeAppointment(session, parent.load(changeException, true),
+                        factory.getContext(), conversionErrors, conversionWarnings);
                 }
             }
             ByteArrayOutputStream bytes = new ByteArrayOutputStream();
@@ -345,7 +363,6 @@ public class AppointmentResource extends CalDAVResource<Appointment> {
              */
             String iCal = new String(bytes.toByteArray(), "UTF-8");
             iCal = Patches.Outgoing.removeEmptyRDates(iCal);
-//            iCal = iCal.replace("@premium", "424242669@devel-mail.netline.de");
             return iCal;
         } catch (final UnsupportedEncodingException e) {
             throw protocolException(e);
@@ -517,7 +534,9 @@ public class AppointmentResource extends CalDAVResource<Appointment> {
                 (Appointment.ABSENT == oldAppointment.getShownAs() || Appointment.TEMPORARY == oldAppointment.getShownAs())) {
                 // don't change "shown as", since iCal maps absent/temporary to reserved
                 updatedAppointment.removeShownAs();
-            } else if (factory.getSession().getUserId() != updatedAppointment.getOrganizerId() &&
+            } else if ((updatedAppointment.containsOrganizerId() && factory.getSession().getUserId() != updatedAppointment.getOrganizerId() ||
+                updatedAppointment.containsOrganizer() && null != updatedAppointment.getOrganizer() &&
+                false == updatedAppointment.getOrganizer().equals(factory.getUser().getMail())) &&
                 isConfirmationChange(oldAppointment, updatedAppointment)) {
                 // don't change "shown as", since iCal clients tend to change the transparency on accept/decline actions of participants
                 updatedAppointment.removeShownAs();
