@@ -51,19 +51,18 @@ package com.openexchange.caldav.servlet;
 
 import java.util.EnumMap;
 import java.util.Map;
-
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
-
 import org.apache.commons.logging.Log;
-
 import com.openexchange.caldav.CaldavProtocol;
 import com.openexchange.caldav.GroupwareCaldavFactory;
 import com.openexchange.caldav.WebdavPostAction;
+import com.openexchange.caldav.action.WebdavMkCalendarAction;
 import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.data.conversion.ical.ICalEmitter;
 import com.openexchange.data.conversion.ical.ICalParser;
 import com.openexchange.folderstorage.FolderService;
+import com.openexchange.freebusy.service.FreeBusyService;
 import com.openexchange.groupware.calendar.AppointmentSqlFactoryService;
 import com.openexchange.groupware.calendar.CalendarCollectionService;
 import com.openexchange.groupware.contexts.Context;
@@ -103,7 +102,7 @@ import com.openexchange.webdav.protocol.helpers.PropertyMixin;
 
 /**
  * The {@link CaldavPerformer} contains all the wiring for caldav actions. This is the central entry point for caldav requests.
- * 
+ *
  * @author <a href="mailto:francisco.laguna@open-xchange.com">Francisco Laguna</a>
  */
 public class CaldavPerformer implements SessionHolder {
@@ -120,7 +119,7 @@ public class CaldavPerformer implements SessionHolder {
 
     /**
      * Gets the instance of {@link InfostorePerformer}.
-     * 
+     *
      * @return The instance of {@link InfostorePerformer}.
      */
     public static CaldavPerformer getInstance() {
@@ -131,7 +130,7 @@ public class CaldavPerformer implements SessionHolder {
     }
 
     public static enum Action {
-        UNLOCK, PROPPATCH, PROPFIND, OPTIONS, MOVE, MKCOL, LOCK, COPY, DELETE, GET, HEAD, PUT, TRACE, REPORT, POST
+        UNLOCK, PROPPATCH, PROPFIND, OPTIONS, MOVE, MKCOL, LOCK, COPY, DELETE, GET, HEAD, PUT, TRACE, REPORT, POST, MKCALENDAR
     }
 
     private final GroupwareCaldavFactory factory;
@@ -159,6 +158,7 @@ public class CaldavPerformer implements SessionHolder {
         WebdavAction put;
         WebdavAction trace;
         WebdavAction post;
+        WebdavAction mkcalendar;
 
         this.factory = new GroupwareCaldavFactory(
             this,
@@ -168,7 +168,8 @@ public class CaldavPerformer implements SessionHolder {
             services.getService(ICalParser.class),
             services.getService(UserService.class),
             services.getService(CalendarCollectionService.class),
-            services.getService(ConfigViewFactory.class));
+            services.getService(ConfigViewFactory.class),
+            services.getService(FreeBusyService.class));
 
         unlock = prepare(new WebdavUnlockAction(), true, true, new WebdavIfAction(0, false, false));
         propPatch = prepare(new WebdavProppatchAction(protocol), true, true, new WebdavExistsAction(), new WebdavIfAction(0, true, false));
@@ -183,6 +184,7 @@ public class CaldavPerformer implements SessionHolder {
         get = prepare(new WebdavGetAction(), true, false, new WebdavExistsAction(), new WebdavIfAction(0, false, false));
         head = prepare(new WebdavHeadAction(), true, true, new WebdavExistsAction(), new WebdavIfAction(0, false, false));
         post = prepare(new WebdavPostAction(factory), true, true, new WebdavIfAction(0, false, false));
+        mkcalendar = prepare(new WebdavMkCalendarAction(), true, true, new WebdavIfAction(0, false, false));
 
         final OXWebdavPutAction oxWebdavPut = new OXWebdavPutAction();
         oxWebdavPut.setSessionHolder(this);
@@ -208,6 +210,7 @@ public class CaldavPerformer implements SessionHolder {
         actions.put(Action.PUT, put);
         actions.put(Action.TRACE, trace);
         actions.put(Action.POST, post);
+        actions.put(Action.MKCALENDAR, mkcalendar);
 
         makeLockNullTolerant();
 
@@ -299,7 +302,7 @@ public class CaldavPerformer implements SessionHolder {
             if (LOG.isDebugEnabled()) {
                 LOG.debug("Executing " + action);
             }
-            
+
             actions.get(action).perform(webdavRequest, webdavResponse);
         } catch (final WebdavProtocolException x) {
             resp.setStatus(x.getStatus());
